package com.example.mob_dev_portfolio.viewModels

import android.app.Application
import android.util.Log
import androidx.lifecycle.AndroidViewModel
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.viewModelScope
import com.example.mob_dev_portfolio.*
import com.example.mob_dev_portfolio.data.UserData
import com.example.mob_dev_portfolio.data.utils.LogInErrors
import com.example.mob_dev_portfolio.data.utils.SignUpErrors
import com.example.mob_dev_portfolio.data.utils.UserType
import com.example.mob_dev_portfolio.ui.LoginViewErrors
import com.example.mob_dev_portfolio.ui.SignUpViewErrors
import kotlinx.coroutines.async
import kotlinx.coroutines.launch

private const val TAG = "AuthViewModel"

class AuthViewModel(application: Application) : AndroidViewModel(application) {

	private val authRepository = (application as ShoppingApplication).authRepository
	private val productsRepository = (application as ShoppingApplication).productsRepository

	private val _userData = MutableLiveData<UserData>()
	val userData: LiveData<UserData> get() = _userData

	private val _signErrorStatus = MutableLiveData<SignUpErrors?>()
	val signErrorStatus: LiveData<SignUpErrors?> get() = _signErrorStatus

	private val _errorStatus = MutableLiveData<SignUpViewErrors>()
	val errorStatus: LiveData<SignUpViewErrors> get() = _errorStatus

	private val _errorStatusLoginFragment = MutableLiveData<LoginViewErrors>()
	val errorStatusLoginFragment: LiveData<LoginViewErrors> get() = _errorStatusLoginFragment

	private val _loginErrorStatus = MutableLiveData<LogInErrors?>()
	val loginErrorStatus: LiveData<LogInErrors?> get() = _loginErrorStatus

	init {
		_errorStatus.value = SignUpViewErrors.NONE
		_errorStatusLoginFragment.value = LoginViewErrors.NONE
		refreshStatus()
	}

	private fun refreshStatus() {
		viewModelScope.launch {
			getCurrUser()
			productsRepository.refreshProducts()
		}
	}

	fun signUpSubmitData(
		name: String,
		mobile: String,
		email: String,
		pwd1: String,
		pwd2: String,
		isAccepted: Boolean,
		isSeller: Boolean
	) {
		if (name.isBlank() || mobile.isBlank() || email.isBlank() || pwd1.isBlank() || pwd2.isBlank()) {
			_errorStatus.value = SignUpViewErrors.ERR_EMPTY
		} else {
			if (pwd1 != pwd2) {
				_errorStatus.value = SignUpViewErrors.ERR_PWD12NS
			} else {
				if (!isAccepted) {
					_errorStatus.value = SignUpViewErrors.ERR_NOT_ACC
				} else {
					var err = ERR_INIT
					if (!isEmailValid(email)) {
						err += ERR_EMAIL
					}
					if (!isPhoneValid(mobile)) {
						err += ERR_MOBILE
					}
					when (err) {
						ERR_INIT -> {
							_errorStatus.value = SignUpViewErrors.NONE
							val uId = getRandomString(32, "44" + mobile.trim(), 6)
							val newData =
								UserData(
									uId,
									name.trim(),
									"+44" + mobile.trim(),
									email.trim(),
									pwd1.trim(),
									ArrayList(),
									ArrayList(),
									ArrayList(),
									ArrayList(),
									if (isSeller) UserType.SELLER.name else UserType.CUSTOMER.name
								)
							_userData.value = newData
							checkUniqueUser(newData)
						}
						(ERR_INIT + ERR_EMAIL) -> _errorStatus.value = SignUpViewErrors.ERR_EMAIL
						(ERR_INIT + ERR_MOBILE) -> _errorStatus.value = SignUpViewErrors.ERR_MOBILE
						(ERR_INIT + ERR_EMAIL + ERR_MOBILE) -> _errorStatus.value =
							SignUpViewErrors.ERR_EMAIL_MOBILE
					}
				}
			}
		}

	}

	private fun checkUniqueUser(uData: UserData) {
		viewModelScope.launch {
			val res = async { authRepository.checkEmailAndMobile(uData.email, uData.mobile, getApplication<ShoppingApplication>().applicationContext) }
			_signErrorStatus.value = res.await()
		}
	}

	fun loginSubmitData(mobile: String, password: String) {
		if (mobile.isBlank() || password.isBlank()) {
			_errorStatusLoginFragment.value = LoginViewErrors.ERR_EMPTY
		} else {
			if (!isPhoneValid(mobile)) {
				_errorStatusLoginFragment.value = LoginViewErrors.ERR_MOBILE
			} else {
				_errorStatusLoginFragment.value = LoginViewErrors.NONE
				logIn("+44" + mobile.trim(), password)
			}
		}
	}

	private fun logIn(phoneNumber: String, pwd: String) {
		viewModelScope.launch {
			val res = async { authRepository.checkLogin(phoneNumber, pwd) }
			_userData.value = res.await()
			if (_userData.value != null) {
				_loginErrorStatus.value = LogInErrors.NONE
			} else {
				_loginErrorStatus.value = LogInErrors.LERR
			}
		}
	}

	private suspend fun getCurrUser() {
		Log.d(TAG, "refreshing data...")
		authRepository.refreshData()
	}
}